"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeconSystem = exports.DeconState = void 0;
const SystemStatus_1 = require("./SystemStatus");
const events_1 = require("../events");
exports.DeconState = {
    Idle: 0x0,
    Enter: 0x1,
    Closed: 0x2,
    Exit: 0x4,
    HeadingUp: 0x8,
};
/**
 * Represents a system responsible for the decontamination doors.
 *
 * See {@link DeconSystemEvents} for events to listen to.
 */
class DeconSystem extends SystemStatus_1.SystemStatus {
    constructor(ship, systemType, data) {
        super(ship, systemType, data);
        this.timer || (this.timer = 0);
        this.state || (this.state = exports.DeconState.Idle);
    }
    Deserialize(reader, spawn) {
        const previousState = this.state;
        this.timer = reader.byte();
        this.state = reader.byte();
        if ((previousState & exports.DeconState.Closed) && !(this.state & exports.DeconState.Closed)) {
            if (this.state & exports.DeconState.Enter) {
                this.emitSync(new events_1.DeconEnterEvent(this.room, this, undefined, undefined, (this.state & exports.DeconState.HeadingUp) > 0));
            }
            else if (this.state & exports.DeconState.Exit) {
                this.emitSync(new events_1.DeconExitEvent(this.room, this, undefined, undefined, (this.state & exports.DeconState.HeadingUp) > 0));
            }
        }
        if (!(previousState & exports.DeconState.Closed) && (this.state & exports.DeconState.Closed)) {
            this.emitSync(new events_1.DeconDoorsCloseEvent(this.room, this, undefined));
        }
        else if ((previousState & exports.DeconState.Closed) && !(this.state & exports.DeconState.Closed)) {
            this.emitSync(new events_1.DeconDoorsOpenEvent(this.room, this, undefined));
        }
        this.dirty = spawn;
    }
    Serialize(writer, spawn) {
        writer.byte(Math.ceil(this.timer));
        writer.byte(this.state);
    }
    async _enterDecon(headingUp, player, message) {
        if ((this.state & exports.DeconState.Enter) && !!(this.state & exports.DeconState.HeadingUp) === headingUp) {
            return;
        }
        const previousHeadingUp = !!exports.DeconState.HeadingUp;
        this.state = exports.DeconState.Enter & ~exports.DeconState.Closed;
        if (headingUp) {
            this.state |= exports.DeconState.HeadingUp;
        }
        this.dirty = true;
        const ev = await this.emit(new events_1.DeconEnterEvent(this.room, this, message, player, headingUp));
        this.emitSync(new events_1.DeconDoorsOpenEvent(this.room, this, undefined));
        if (ev.reverted) {
            this.state &= ~exports.DeconState.Enter;
            if (previousHeadingUp) {
                this.state |= exports.DeconState.HeadingUp;
            }
            else {
                this.state &= ~exports.DeconState.HeadingUp;
            }
        }
        else {
            this.timer = 3;
        }
    }
    async enterDecon(headingUp) {
        if (this.ship.canBeManaged()) {
            await this._enterDecon(headingUp, this.room.myPlayer, undefined);
        }
        else {
            await this._sendRepair(headingUp ? 1 : 2);
        }
    }
    async _exitDecon(headingUp, player, message) {
        if ((this.state & exports.DeconState.Exit) && !!(this.state & exports.DeconState.HeadingUp) === headingUp) {
            return;
        }
        const previousHeadingUp = !!exports.DeconState.HeadingUp;
        this.state = exports.DeconState.Exit;
        if (headingUp) {
            this.state |= exports.DeconState.HeadingUp;
        }
        this.dirty = true;
        const ev = await this.emit(new events_1.DeconExitEvent(this.room, this, message, player, headingUp));
        this.emitSync(new events_1.DeconDoorsCloseEvent(this.room, this, undefined));
        if (ev.reverted) {
            this.state &= ~exports.DeconState.Exit;
            if (previousHeadingUp) {
                this.state |= exports.DeconState.HeadingUp;
            }
            else {
                this.state &= ~exports.DeconState.HeadingUp;
            }
        }
        else {
            this.timer = 3;
        }
    }
    async exitDecon(headingUp) {
        if (this.ship.canBeManaged()) {
            await this._exitDecon(headingUp, this.room.myPlayer, undefined);
        }
        else {
            await this._sendRepair(headingUp ? 3 : 4);
        }
    }
    async HandleRepair(player, amount, rpc) {
        if (this.state !== exports.DeconState.Idle)
            return;
        switch (amount) {
            case 1:
                await this._enterDecon(true, player, rpc);
                break;
            case 2:
                await this._enterDecon(false, player, rpc);
                break;
            case 3:
                await this._exitDecon(true, player, rpc);
                break;
            case 4:
                await this._exitDecon(false, player, rpc);
                break;
        }
        this.timer = 3;
        this.dirty = true;
    }
    Detoriorate(delta) {
        const previous = this.timer;
        this.timer -= delta;
        if (this.timer < 0) {
            this.timer = 0;
        }
        if (this.timer !== previous) {
            if (this.timer <= 0) {
                if (this.state & exports.DeconState.Enter) {
                    this.state = (this.state & ~exports.DeconState.Enter) | exports.DeconState.Closed;
                    this.timer = 3;
                    this.emitSync(new events_1.DeconDoorsCloseEvent(this.room, this, undefined));
                }
                else if (this.state & exports.DeconState.Closed) {
                    this.state = (this.state & ~exports.DeconState.Closed) | exports.DeconState.Exit;
                    this.timer = 3;
                    this.emitSync(new events_1.DeconDoorsOpenEvent(this.room, this, undefined));
                }
                else if (this.state & exports.DeconState.Exit) {
                    this.state = exports.DeconState.Idle;
                }
            }
        }
    }
}
exports.DeconSystem = DeconSystem;
//# sourceMappingURL=DeconSystem.js.map